@@ -18,6 +18,7 @@ import logging
 # Import salt libs
 import salt.crypt
 import salt.utils
+import salt.client
 import salt.exceptions
 import salt.utils.event
 import salt.daemons.masterapi
@@ -99,12 +100,13 @@ class KeyCLI(object):
                 'key',
                 self.opts)
 
-    def accept(self, match, include_rejected=False):
+    def accept(self, match, include_rejected=False, include_denied=False):
         '''
         Accept the keys matched
 
         :param str match: A string to match against. i.e. 'web*'
         :param bool include_rejected: Whether or not to accept a matched key that was formerly rejected
+        :param bool include_denied: Whether or not to accept a matched key that was formerly denied
         '''
         def _print_accepted(matches, after_match):
             if self.key.ACC in after_match:
@@ -122,10 +124,14 @@ class KeyCLI(object):
             keys[self.key.PEND] = matches[self.key.PEND]
         if include_rejected and bool(matches.get(self.key.REJ)):
             keys[self.key.REJ] = matches[self.key.REJ]
+        if include_denied and bool(matches.get(self.key.DEN)):
+            keys[self.key.DEN] = matches[self.key.DEN]
         if not keys:
             msg = (
-                'The key glob {0!r} does not match any unaccepted {1}keys.'
-                .format(match, 'or rejected ' if include_rejected else '')
+                'The key glob \'{0}\' does not match any unaccepted{1} keys.'
+                .format(match, (('', ' or denied'),
+                                (' or rejected', ', rejected or denied')
+                               )[include_rejected][include_denied])
             )
             print(msg)
             raise salt.exceptions.SaltSystemExit(code=1)
@@ -144,7 +150,8 @@ class KeyCLI(object):
                     matches,
                     self.key.accept(
                         match_dict=keys,
-                        include_rejected=include_rejected
+                        include_rejected=include_rejected,
+                        include_denied=include_denied
                     )
                 )
         else:
@@ -157,17 +164,19 @@ class KeyCLI(object):
                 matches,
                 self.key.accept(
                     match_dict=keys,
-                    include_rejected=include_rejected
+                    include_rejected=include_rejected,
+                    include_denied=include_denied
                 )
             )
 
-    def accept_all(self, include_rejected=False):
+    def accept_all(self, include_rejected=False, include_denied=False):
         '''
         Accept all keys
 
         :param bool include_rejected: Whether or not to accept a matched key that was formerly rejected
+        :param bool include_denied: Whether or not to accept a matched key that was formerly denied
         '''
-        self.accept('*', include_rejected=include_rejected)
+        self.accept('*', include_rejected=include_rejected, include_denied=include_denied)
 
     def delete(self, match):
         '''
@@ -189,7 +198,7 @@ class KeyCLI(object):
         matches = self.key.name_match(match)
         if not matches:
             print(
-                'The key glob {0!r} does not match any accepted, unaccepted '
+                'The key glob \'{0}\' does not match any accepted, unaccepted '
                 'or rejected keys.'.format(match)
             )
             raise salt.exceptions.SaltSystemExit(code=1)
@@ -206,7 +215,7 @@ class KeyCLI(object):
             if veri.lower().startswith('y'):
                 _print_deleted(
                     matches,
-                    self.key.delete_key(match_dict=matches)
+                    self.key.delete_key(match_dict=matches, revoke_auth=True)
                 )
         else:
             print('Deleting the following keys:')
@@ -225,12 +234,15 @@ class KeyCLI(object):
         '''
         self.delete('*')
 
-    def reject(self, match, include_accepted=False):
+    def reject(self, match, include_accepted=False, include_denied=False):
         '''
         Reject the matched keys
 
         :param str match: A string to match against. i.e. 'web*'
-        :param bool include_accepted: Whether or not to accept a matched key that was formerly accepted
+        :param bool include_accepted: Whether or not to reject a matched key
+        that was formerly accepted
+        :param bool include_denied: Whether or not to reject a matched key
+        that was formerly denied
         '''
         def _print_rejected(matches, after_match):
             if self.key.REJ in after_match:
@@ -248,10 +260,14 @@ class KeyCLI(object):
             keys[self.key.PEND] = matches[self.key.PEND]
         if include_accepted and bool(matches.get(self.key.ACC)):
             keys[self.key.ACC] = matches[self.key.ACC]
+        if include_denied and bool(matches.get(self.key.DEN)):
+            keys[self.key.DEN] = matches[self.key.DEN]
         if not keys:
-            msg = 'The key glob {0!r} does not match any {1} keys.'.format(
+            msg = 'The key glob \'{0}\' does not match any {1} keys.'.format(
                 match,
-                'accepted or unaccepted' if include_accepted else 'unaccepted'
+                (('unaccepted', 'unaccepted or denied'),
+                 ('accepted or unaccepted', 'accepted, unaccepted or denied')
+                )[include_accepted][include_denied]
             )
             print(msg)
             return
@@ -268,17 +284,19 @@ class KeyCLI(object):
             matches,
             self.key.reject(
                 match_dict=matches,
-                include_accepted=include_accepted
+                include_accepted=include_accepted,
+                include_denied=include_denied
             )
         )
 
-    def reject_all(self, include_accepted=False):
+    def reject_all(self, include_accepted=False, include_denied=False):
         '''
         Reject all keys
 
-        :param bool include_accepted: Whether or not to accept a matched key that was formerly accepted
+        :param bool include_accepted: Whether or not to reject a matched key that was formerly accepted
+        :param bool include_denied: Whether or not to reject a matched key that was formerly denied
         '''
-        self.reject('*', include_accepted=include_accepted)
+        self.reject('*', include_accepted=include_accepted, include_denied=include_denied)
 
     def print_key(self, match):
         '''
@@ -410,17 +428,25 @@ class KeyCLI(object):
         elif self.opts['accept']:
             self.accept(
                 self.opts['accept'],
-                include_rejected=self.opts['include_all']
+                include_rejected=self.opts['include_all'] or self.opts['include_rejected'],
+                include_denied=self.opts['include_denied']
             )
         elif self.opts['accept_all']:
-            self.accept_all(include_rejected=self.opts['include_all'])
+            self.accept_all(
+                include_rejected=self.opts['include_all'] or self.opts['include_rejected'],
+                include_denied=self.opts['include_denied']
+            )
         elif self.opts['reject']:
             self.reject(
                 self.opts['reject'],
-                include_accepted=self.opts['include_all']
+                include_accepted=self.opts['include_all'] or self.opts['include_accepted'],
+                include_denied=self.opts['include_denied']
             )
         elif self.opts['reject_all']:
-            self.reject_all(include_accepted=self.opts['include_all'])
+            self.reject_all(
+                include_accepted=self.opts['include_all'] or self.opts['include_accepted'],
+                include_denied=self.opts['include_denied']
+            )
         elif self.opts['delete']:
             self.delete(self.opts['delete'])
         elif self.opts['delete_all']:
@@ -464,11 +490,11 @@ class MultiKeyCLI(KeyCLI):
     def list_all(self):
         self._call_all('list_all')
 
-    def accept(self, match, include_rejected=False):
-        self._call_all('accept', match, include_rejected)
+    def accept(self, match, include_rejected=False, include_denied=False):
+        self._call_all('accept', match, include_rejected, include_denied)
 
-    def accept_all(self, include_rejected=False):
-        self._call_all('accept_all', include_rejected)
+    def accept_all(self, include_rejected=False, include_denied=False):
+        self._call_all('accept_all', include_rejected, include_denied)
 
     def delete(self, match):
         self._call_all('delete', match)
@@ -476,11 +502,11 @@ class MultiKeyCLI(KeyCLI):
     def delete_all(self):
         self._call_all('delete_all')
 
-    def reject(self, match, include_accepted=False):
-        self._call_all('reject', match, include_accepted)
+    def reject(self, match, include_accepted=False, include_denied=False):
+        self._call_all('reject', match, include_accepted, include_denied)
 
-    def reject_all(self, include_accepted=False):
-        self._call_all('reject_all', include_accepted)
+    def reject_all(self, include_accepted=False, include_denied=False):
+        self._call_all('reject_all', include_accepted, include_denied)
 
     def print_key(self, match):
         self._call_all('print_key', match)
@@ -738,7 +764,7 @@ class Key(object):
                     ret[status][key] = fp_.read()
         return ret
 
-    def accept(self, match=None, match_dict=None, include_rejected=False):
+    def accept(self, match=None, match_dict=None, include_rejected=False, include_denied=False):
         '''
         Accept public keys. If "match" is passed, it is evaluated as a glob.
         Pre-gathered matches can also be passed via "match_dict".
@@ -752,6 +778,8 @@ class Key(object):
         keydirs = [self.PEND]
         if include_rejected:
             keydirs.append(self.REJ)
+        if include_denied:
+            keydirs.append(self.DEN)
         for keydir in keydirs:
             for key in matches.get(keydir, []):
                 try:
@@ -801,7 +829,11 @@ class Key(object):
                 pass
         return self.list_keys()
 
-    def delete_key(self, match=None, match_dict=None, preserve_minions=False):
+    def delete_key(self,
+                    match=None,
+                    match_dict=None,
+                    preserve_minions=False,
+                    revoke_auth=False):
         '''
         Delete public keys. If "match" is passed, it is evaluated as a glob.
         Pre-gathered matches can also be passed via "match_dict".
@@ -817,6 +849,19 @@ class Key(object):
         for status, keys in six.iteritems(matches):
             for key in keys:
                 try:
+                    if revoke_auth:
+                        if self.opts.get('rotate_aes_key') is False:
+                            print('Immediate auth revocation specified but AES key rotation not allowed. '
+                                     'Minion will not be disconnected until the master AES key is rotated.')
+                        else:
+                            try:
+                                client = salt.client.get_local_client(mopts=self.opts)
+                                client.cmd_async(key, 'saltutil.revoke_auth')
+                            except salt.exceptions.SaltClientError:
+                                print('Cannot contact Salt master. '
+                                      'Connection for {0} will remain up until '
+                                      'master AES key is rotated or auth is revoked '
+                                      'with \'saltutil.revoke_auth\'.'.format(key))
                     os.remove(os.path.join(self.opts['pki_dir'], status, key))
                     eload = {'result': True,
                              'act': 'delete',
@@ -824,7 +869,11 @@ class Key(object):
                     self.event.fire_event(eload, tagify(prefix='key'))
                 except (OSError, IOError):
                     pass
-        self.check_minion_cache(preserve_minions=matches.get('minions', []))
+        if preserve_minions:
+            preserve_minions_list = matches.get('minions', [])
+        else:
+            preserve_minions_list = []
+        self.check_minion_cache(preserve_minions=preserve_minions_list)
         if self.opts.get('rotate_aes_key'):
             salt.crypt.dropfile(self.opts['cachedir'], self.opts['user'])
         return (
@@ -832,6 +881,24 @@ class Key(object):
             else self.dict_match(matches)
         )
 
+    def delete_den(self):
+        '''
+        Delete all denied keys
+        '''
+        keys = self.list_keys()
+        for status, keys in six.iteritems(self.list_keys()):
+            for key in keys[self.DEN]:
+                try:
+                    os.remove(os.path.join(self.opts['pki_dir'], status, key))
+                    eload = {'result': True,
+                                 'act': 'delete',
+                                 'id': key}
+                    self.event.fire_event(eload, tagify(prefix='key'))
+                except (OSError, IOError):
+                    pass
+        self.check_minion_cache()
+        return self.list_keys()
+
     def delete_all(self):
         '''
         Delete all keys
@@ -851,7 +918,7 @@ class Key(object):
             salt.crypt.dropfile(self.opts['cachedir'], self.opts['user'])
         return self.list_keys()
 
-    def reject(self, match=None, match_dict=None, include_accepted=False):
+    def reject(self, match=None, match_dict=None, include_accepted=False, include_denied=False):
         '''
         Reject public keys. If "match" is passed, it is evaluated as a glob.
         Pre-gathered matches can also be passed via "match_dict".
@@ -865,6 +932,8 @@ class Key(object):
         keydirs = [self.PEND]
         if include_accepted:
             keydirs.append(self.ACC)
+        if include_denied:
+            keydirs.append(self.DEN)
         for keydir in keydirs:
             for key in matches.get(keydir, []):
                 try:
@@ -933,22 +1002,22 @@ class Key(object):
                     path = os.path.join(self.opts['pki_dir'], key)
                 else:
                     path = os.path.join(self.opts['pki_dir'], status, key)
-                ret[status][key] = salt.utils.pem_finger(path)
+                ret[status][key] = salt.utils.pem_finger(path, sum_type=self.opts['hash_type'])
         return ret
 
     def finger_all(self):
         '''
-        Return fingerprins for all keys
+        Return fingerprints for all keys
         '''
         ret = {}
-        for status, keys in six.iteritems(self.list_keys()):
+        for status, keys in six.iteritems(self.all_keys()):
             ret[status] = {}
             for key in keys:
                 if status == 'local':
                     path = os.path.join(self.opts['pki_dir'], key)
                 else:
                     path = os.path.join(self.opts['pki_dir'], status, key)
-                ret[status][key] = salt.utils.pem_finger(path)
+                ret[status][key] = salt.utils.pem_finger(path, sum_type=self.opts['hash_type'])
         return ret
 
 
@@ -1168,7 +1237,7 @@ class RaetKey(Key):
                 ret[status][key] = self._get_key_str(key, status)
         return ret
 
-    def accept(self, match=None, match_dict=None, include_rejected=False):
+    def accept(self, match=None, match_dict=None, include_rejected=False, include_denied=False):
         '''
         Accept public keys. If "match" is passed, it is evaluated as a glob.
         Pre-gathered matches can also be passed via "match_dict".
@@ -1182,6 +1251,8 @@ class RaetKey(Key):
         keydirs = [self.PEND]
         if include_rejected:
             keydirs.append(self.REJ)
+        if include_denied:
+            keydirs.append(self.DEN)
         for keydir in keydirs:
             for key in matches.get(keydir, []):
                 try:
@@ -1223,7 +1294,11 @@ class RaetKey(Key):
                 pass
         return self.list_keys()
 
-    def delete_key(self, match=None, match_dict=None, preserve_minions=False):
+    def delete_key(self,
+                   match=None,
+                   match_dict=None,
+                   preserve_minions=False,
+                   revoke_auth=False):
         '''
         Delete public keys. If "match" is passed, it is evaluated as a glob.
         Pre-gathered matches can also be passed via "match_dict".
@@ -1236,6 +1311,19 @@ class RaetKey(Key):
             matches = {}
         for status, keys in six.iteritems(matches):
             for key in keys:
+                if revoke_auth:
+                    if self.opts.get('rotate_aes_key') is False:
+                        print('Immediate auth revocation specified but AES key rotation not allowed. '
+                                 'Minion will not be disconnected until the master AES key is rotated.')
+                    else:
+                        try:
+                            client = salt.client.get_local_client(mopts=self.opts)
+                            client.cmd_async(key, 'saltutil.revoke_auth')
+                        except salt.exceptions.SaltClientError:
+                            print('Cannot contact Salt master. '
+                                  'Connection for {0} will remain up until '
+                                  'master AES key is rotated or auth is revoked '
+                                  'with \'saltutil.revoke_auth\'.'.format(key))
                 try:
                     os.remove(os.path.join(self.opts['pki_dir'], status, key))
                 except (OSError, IOError):
@@ -1259,7 +1347,7 @@ class RaetKey(Key):
         self.check_minion_cache()
         return self.list_keys()
 
-    def reject(self, match=None, match_dict=None, include_accepted=False):
+    def reject(self, match=None, match_dict=None, include_accepted=False, include_denied=False):
         '''
         Reject public keys. If "match" is passed, it is evaluated as a glob.
         Pre-gathered matches can also be passed via "match_dict".
@@ -1273,6 +1361,8 @@ class RaetKey(Key):
         keydirs = [self.PEND]
         if include_accepted:
             keydirs.append(self.ACC)
+        if include_denied:
+            keydirs.append(self.DEN)
         for keydir in keydirs:
             for key in matches.get(keydir, []):
                 try:
