@@ -31,7 +31,7 @@ def __virtual__():
     '''
     # Disable on Windows, a specific file module exists:
     if salt.utils.is_windows():
-        return False
+        return (False, 'The mount module cannot be loaded: not a POSIX-like system.')
     return True
 
 
@@ -80,11 +80,11 @@ def _active_mountinfo(ret):
                              'major': device[0],
                              'minor': device[1],
                              'root': comps[3],
-                             'opts': comps[5].split(','),
+                             'opts': _resolve_user_group_names(comps[5].split(',')),
                              'fstype': comps[_sep + 1],
                              'device': device_name,
                              'alt_device': _list.get(comps[4], None),
-                             'superopts': comps[_sep + 3].split(','),
+                             'superopts': _resolve_user_group_names(comps[_sep + 3].split(',')),
                              'device_uuid': device_uuid,
                              'device_label': device_label}
     return ret
@@ -106,7 +106,7 @@ def _active_mounts(ret):
             ret[comps[1]] = {'device': comps[0],
                              'alt_device': _list.get(comps[1], None),
                              'fstype': comps[2],
-                             'opts': comps[3].split(',')}
+                             'opts': _resolve_user_group_names(comps[3].split(','))}
     return ret
 
 
@@ -118,7 +118,7 @@ def _active_mounts_freebsd(ret):
         comps = re.sub(r"\s+", " ", line).split()
         ret[comps[1]] = {'device': comps[0],
                          'fstype': comps[2],
-                         'opts': comps[3].split(',')}
+                         'opts': _resolve_user_group_names(comps[3].split(','))}
     return ret
 
 
@@ -130,7 +130,7 @@ def _active_mounts_solaris(ret):
         comps = re.sub(r"\s+", " ", line).split()
         ret[comps[2]] = {'device': comps[0],
                          'fstype': comps[4],
-                         'opts': comps[5].split('/')}
+                         'opts': _resolve_user_group_names(comps[5].split('/'))}
     return ret
 
 
@@ -146,14 +146,14 @@ def _active_mounts_openbsd(ret):
         if len(parens) > 1:
             ret[comps[3]] = {'device': comps[0],
                          'fstype': comps[5],
-                         'opts': parens[1].split(", "),
+                         'opts': _resolve_user_group_names(parens[1].split(", ")),
                          'major': str(nod[4].strip(",")),
                          'minor': str(nod[5]),
                          'device_uuid': parens[0]}
         else:
             ret[comps[2]] = {'device': comps[0],
                             'fstype': comps[4],
-                            'opts': parens[0].split(", ")}
+                            'opts': _resolve_user_group_names(parens[1].split(", "))}
     return ret
 
 
@@ -166,10 +166,29 @@ def _active_mounts_darwin(ret):
         parens = re.findall(r'\((.*?)\)', line, re.DOTALL)[0].split(", ")
         ret[comps[2]] = {'device': comps[0],
                          'fstype': parens[0],
-                         'opts': parens[1:]}
+                         'opts': _resolve_user_group_names(parens[1:])}
     return ret
 
 
+def _resolve_user_group_names(opts):
+    '''
+    Resolve user and group names in related opts
+    '''
+    name_id_opts = {'uid': 'user.info',
+                    'gid': 'group.info'}
+    for ind, opt in enumerate(opts):
+        if opt.split('=')[0] in name_id_opts:
+            _givenid = opt.split('=')[1]
+            _param = opt.split('=')[0]
+            _id = _givenid
+            if not re.match('[0-9]+$', _givenid):
+                _info = __salt__[name_id_opts[_param]](_givenid)
+                if _info and _param in _info:
+                    _id = _info[_param]
+            opts[ind] = _param + '=' + str(_id)
+    return opts
+
+
 def active(extended=False):
     '''
     List the active mounts.
@@ -294,6 +313,9 @@ def fstab(config='/etc/fstab'):
                     _fstab_entry.compatibility_keys)
 
                 entry['opts'] = entry['opts'].split(',')
+                while entry['name'] in ret:
+                    entry['name'] += '_'
+
                 ret[entry.pop('name')] = entry
             except _fstab_entry.ParseError:
                 pass
@@ -684,7 +706,7 @@ def automaster(config='/etc/auto_salt'):
     return ret
 
 
-def mount(name, device, mkmnt=False, fstype='', opts='defaults', user=None):
+def mount(name, device, mkmnt=False, fstype='', opts='defaults', user=None, util='mount'):
     '''
     Mount a device
 
@@ -694,6 +716,18 @@ def mount(name, device, mkmnt=False, fstype='', opts='defaults', user=None):
 
         salt '*' mount.mount /mnt/foo /dev/sdz1 True
     '''
+    if util != 'mount':
+        # This functionality used to live in img.mount_image
+        if util == 'guestfs':
+            return __salt__['guestfs.mount'](name, root=device)
+        elif util == 'qemu_nbd':
+            mnt = __salt__['qemu_nbd.init'](name, device)
+            if not mnt:
+                return False
+            first = next(six.iterkeys(mnt))
+            __context__['img.mnt_{0}'.format(first)] = mnt
+            return first
+        return False
 
     # Darwin doesn't expect defaults when mounting without other options
     if 'defaults' in opts and __grains__['os'] in ['MacOS', 'Darwin']:
@@ -762,7 +796,7 @@ def remount(name, device, mkmnt=False, fstype='', opts='defaults', user=None):
     return mount(name, device, mkmnt, fstype, opts, user=user)
 
 
-def umount(name, device=None, user=None):
+def umount(name, device=None, user=None, util='mount'):
     '''
     Attempt to unmount a device by specifying the directory it is mounted on
 
@@ -777,6 +811,13 @@ def umount(name, device=None, user=None):
 
         salt '*' mount.umount /mnt/foo /dev/xvdc1
     '''
+    if util != 'mount':
+        # This functionality used to live in img.umount_image
+        if 'qemu_nbd.clear' in __salt__:
+            if 'img.mnt_{0}'.format(name) in __context__:
+                __salt__['qemu_nbd.clear'](__context__['img.mnt_{0}'.format(name)])
+                return
+
     mnts = active()
     if name not in mnts:
         return "{0} does not have anything mounted".format(name)
