@@ -15,9 +15,11 @@ from __future__ import absolute_import
 import copy
 import logging
 import re
+import os.path
 
 # Import salt libs
 import salt.utils
+import salt.utils.itertools
 from salt.exceptions import CommandExecutionError, MinionError
 
 # Import 3rd-party libs
@@ -33,9 +35,9 @@ def __virtual__():
     '''
     Set the virtual pkg module if the os is Arch
     '''
-    if __grains__['os'] in ('Arch', 'Arch ARM', 'ManjaroLinux'):
+    if __grains__['os'] in ('Arch', 'Arch ARM', 'Antergos', 'ManjaroLinux'):
         return __virtualname__
-    return False
+    return (False, 'The pacman module could not be loaded: unsupported OS family.')
 
 
 def _list_removed(old, new):
@@ -74,10 +76,16 @@ def latest_version(*names, **kwargs):
     # Initialize the dict with empty strings
     for name in names:
         ret[name] = ''
-    cmd = 'pacman -Sp --needed --print-format "%n %v" ' \
-          '{0}'.format(' '.join(names))
-    out = __salt__['cmd.run_stdout'](cmd, output_loglevel='trace')
-    for line in out.splitlines():
+    cmd = ['pacman', '-Sp', '--needed', '--print-format', '%n %v']
+    cmd.extend(names)
+
+    if 'root' in kwargs:
+        cmd.extend(('-r', kwargs['root']))
+
+    out = __salt__['cmd.run_stdout'](cmd,
+                                     output_loglevel='trace',
+                                     python_shell=False)
+    for line in salt.utils.itertools.split(out, '\n'):
         try:
             name, version_num = line.split()
             # Only add to return dict if package is in the list of packages
@@ -118,7 +126,7 @@ def upgrade_available(name):
     return latest_version(name) != ''
 
 
-def list_upgrades(refresh=False):
+def list_upgrades(refresh=False, root=None):
     '''
     List all available package upgrades on this system
 
@@ -129,14 +137,17 @@ def list_upgrades(refresh=False):
         salt '*' pkg.list_upgrades
     '''
     upgrades = {}
-    options = ['-S', '-p', '-u', '--print-format "%n %v"']
+    cmd = ['pacman', '-S', '-p', '-u', '--print-format', '%n %v']
+
+    if root is not None:
+        cmd.extend(('-r', root))
 
     if refresh:
-        options.append('-y')
+        cmd.append('-y')
 
-    cmd = ('pacman {0}').format(' '.join(options))
-
-    call = __salt__['cmd.run_all'](cmd, output_loglevel='trace')
+    call = __salt__['cmd.run_all'](cmd,
+                                   output_loglevel='trace',
+                                   python_shell=False)
 
     if call['retcode'] != 0:
         comment = ''
@@ -144,13 +155,13 @@ def list_upgrades(refresh=False):
             comment += call['stderr']
         if 'stdout' in call:
             comment += call['stdout']
-        raise CommandExecutionError(
-            '{0}'.format(comment)
-        )
+        if comment:
+            comment = ': ' + comment
+        raise CommandExecutionError('Error listing upgrades' + comment)
     else:
         out = call['stdout']
 
-    for line in iter(out.splitlines()):
+    for line in salt.utils.itertools.split(out, '\n'):
         comps = line.split(' ')
         if len(comps) != 2:
             continue
@@ -200,17 +211,21 @@ def list_pkgs(versions_as_list=False, **kwargs):
             __salt__['pkg_resource.stringify'](ret)
             return ret
 
-    cmd = 'pacman -Q'
+    cmd = ['pacman', '-Q']
+
+    if 'root' in kwargs:
+        cmd.extend(('-r', kwargs['root']))
+
     ret = {}
-    out = __salt__['cmd.run'](cmd, output_loglevel='trace')
-    for line in out.splitlines():
+    out = __salt__['cmd.run'](cmd, output_loglevel='trace', python_shell=False)
+    for line in salt.utils.itertools.split(out, '\n'):
         if not line:
             continue
         try:
             name, version_num = line.split()[0:2]
         except ValueError:
             log.error('Problem parsing pacman -Q: Unexpected formatting in '
-                      'line: "{0}"'.format(line))
+                      'line: \'{0}\''.format(line))
         else:
             __salt__['pkg_resource.add_pkg'](ret, name, version_num)
 
@@ -221,7 +236,7 @@ def list_pkgs(versions_as_list=False, **kwargs):
     return ret
 
 
-def refresh_db():
+def refresh_db(root=None):
     '''
     Just run a ``pacman -Sy``, return a dict::
 
@@ -233,22 +248,27 @@ def refresh_db():
 
         salt '*' pkg.refresh_db
     '''
-    cmd = 'LANG=C pacman -Sy'
+    cmd = ['pacman', '-Sy']
+
+    if root is not None:
+        cmd.extend(('-r', root))
+
     ret = {}
-    call = __salt__['cmd.run_all'](cmd, output_loglevel='trace',
-            python_shell=True)
+    call = __salt__['cmd.run_all'](cmd,
+                                   output_loglevel='trace',
+                                   env={'LANG': 'C'},
+                                   python_shell=False)
     if call['retcode'] != 0:
         comment = ''
         if 'stderr' in call:
-            comment += call['stderr']
-
+            comment += ': ' + call['stderr']
         raise CommandExecutionError(
-            '{0}'.format(comment)
+            'Error refreshing package database' + comment
         )
     else:
         out = call['stdout']
 
-    for line in out.splitlines():
+    for line in salt.utils.itertools.split(out, '\n'):
         if line.strip().startswith('::'):
             continue
         if not line:
@@ -341,16 +361,17 @@ def install(name=None,
     version_num = kwargs.get('version')
     if version_num:
         if pkgs is None and sources is None:
-            # Allow "version" to work for single package target
+            # Allow 'version' to work for single package target
             pkg_params = {name: version_num}
         else:
-            log.warning('"version" parameter will be ignored for multiple '
+            log.warning('\'version\' parameter will be ignored for multiple '
                         'package targets')
 
+    if 'root' in kwargs:
+        pkg_params['-r'] = kwargs['root']
+
     if pkg_type == 'file':
-        cmd = 'pacman -U --noprogressbar --noconfirm ' \
-              '{0}'.format(' '.join(pkg_params))
-        targets = pkg_params
+        cmd = ['pacman', '-U', '--noprogressbar', '--noconfirm'] + pkg_params
     elif pkg_type == 'repository':
         targets = []
         problems = []
@@ -368,8 +389,8 @@ def install(name=None,
                     prefix = prefix or '='
                     targets.append('{0}{1}{2}'.format(param, prefix, verstr))
                 else:
-                    msg = 'Invalid version string "{0}" for package ' \
-                          '"{1}"'.format(version_num, name)
+                    msg = 'Invalid version string \'{0}\' for package ' \
+                          '\'{1}\''.format(version_num, name)
                     problems.append(msg)
         if problems:
             for problem in problems:
@@ -381,16 +402,34 @@ def install(name=None,
         if salt.utils.is_true(sysupgrade):
             options.append('-u')
 
-        cmd = 'pacman -S "{0}"'.format('" "'.join(options+targets))
+        cmd = ['pacman', '-S'] + options + targets
 
     old = list_pkgs()
-    __salt__['cmd.run'](cmd, output_loglevel='trace')
+    out = __salt__['cmd.run_all'](
+        cmd,
+        output_loglevel='trace',
+        python_shell=False
+    )
+
+    if out['retcode'] != 0 and out['stderr']:
+        errors = [out['stderr']]
+    else:
+        errors = []
+
     __context__.pop('pkg.list_pkgs', None)
     new = list_pkgs()
-    return salt.utils.compare_dicts(old, new)
+    ret = salt.utils.compare_dicts(old, new)
+
+    if errors:
+        raise CommandExecutionError(
+            'Problem encountered installing package(s)',
+            info={'errors': errors, 'changes': ret}
+        )
+
+    return ret
 
 
-def upgrade(refresh=False, **kwargs):
+def upgrade(refresh=False, root=None, **kwargs):
     '''
     Run a full system upgrade, a pacman -Syu
 
@@ -410,24 +449,31 @@ def upgrade(refresh=False, **kwargs):
     '''
     ret = {'changes': {},
            'result': True,
-           'comment': '',
-           }
+           'comment': ''}
 
     old = list_pkgs()
-    cmd = 'pacman -Su --noprogressbar --noconfirm'
+    cmd = ['pacman', '-Su', '--noprogressbar', '--noconfirm']
+
     if salt.utils.is_true(refresh):
-        cmd += ' -y'
-    call = __salt__['cmd.run_all'](cmd, output_loglevel='trace')
+        cmd.append('-y')
+
+    if root is not None:
+        cmd.extend(('-r', root))
+
+    call = __salt__['cmd.run_all'](cmd,
+                                   output_loglevel='trace',
+                                   python_shell=False,
+                                   redirect_stderr=True)
+
     if call['retcode'] != 0:
         ret['result'] = False
-        if 'stderr' in call:
-            ret['comment'] += call['stderr']
-        if 'stdout' in call:
-            ret['comment'] += call['stdout']
-    else:
-        __context__.pop('pkg.list_pkgs', None)
-        new = list_pkgs()
-        ret['changes'] = salt.utils.compare_dicts(old, new)
+        if call['stdout']:
+            ret['comment'] = call['stdout']
+
+    __context__.pop('pkg.list_pkgs', None)
+    new = list_pkgs()
+    ret['changes'] = salt.utils.compare_dicts(old, new)
+
     return ret
 
 
@@ -445,16 +491,37 @@ def _uninstall(action='remove', name=None, pkgs=None, **kwargs):
     targets = [x for x in pkg_params if x in old]
     if not targets:
         return {}
-    remove_arg = '-Rsc' if action == 'purge' else '-R'
-    cmd = (
-        'pacman {0} '
-        '--noprogressbar '
-        '--noconfirm {1}'
-    ).format(remove_arg, ' '.join(targets))
-    __salt__['cmd.run'](cmd, output_loglevel='trace')
+
+    cmd = ['pacman',
+           '-Rsc' if action == 'purge' else '-R',
+           '--noprogressbar',
+           '--noconfirm'] + targets
+
+    if 'root' in kwargs:
+        cmd.extend(('-r', kwargs['root']))
+
+    out = __salt__['cmd.run_all'](
+        cmd,
+        output_loglevel='trace',
+        python_shell=False
+    )
+
+    if out['retcode'] != 0 and out['stderr']:
+        errors = [out['stderr']]
+    else:
+        errors = []
+
     __context__.pop('pkg.list_pkgs', None)
     new = list_pkgs()
-    return salt.utils.compare_dicts(old, new)
+    ret = salt.utils.compare_dicts(old, new)
+
+    if errors:
+        raise CommandExecutionError(
+            'Problem encountered removing package(s)',
+            info={'errors': errors, 'changes': ret}
+        )
+
+    return ret
 
 
 def remove(name=None, pkgs=None, **kwargs):
@@ -534,9 +601,16 @@ def file_list(*packages):
     '''
     errors = []
     ret = []
-    cmd = 'pacman -Ql {0}'.format(' '.join(packages))
-    out = __salt__['cmd.run'](cmd, output_loglevel='trace')
-    for line in out.splitlines():
+    cmd = ['pacman', '-Ql']
+
+    if len(packages) > 0 and os.path.exists(packages[0]):
+        packages = list(packages)
+        cmd.extend(('-r', packages.pop(0)))
+
+    cmd.extend(packages)
+
+    out = __salt__['cmd.run'](cmd, output_loglevel='trace', python_shell=False)
+    for line in salt.utils.itertools.split(out, '\n'):
         if line.startswith('error'):
             errors.append(line)
         else:
@@ -561,9 +635,16 @@ def file_dict(*packages):
     '''
     errors = []
     ret = {}
-    cmd = 'pacman -Ql {0}'.format(' '.join(packages))
-    out = __salt__['cmd.run'](cmd, output_loglevel='trace')
-    for line in out.splitlines():
+    cmd = ['pacman', '-Ql']
+
+    if len(packages) > 0 and os.path.exists(packages[0]):
+        packages = list(packages)
+        cmd.extend(('-r', packages.pop(0)))
+
+    cmd.extend(packages)
+
+    out = __salt__['cmd.run'](cmd, output_loglevel='trace', python_shell=False)
+    for line in salt.utils.itertools.split(out, '\n'):
         if line.startswith('error'):
             errors.append(line)
         else:
@@ -594,9 +675,11 @@ def owner(*paths):
     if not paths:
         return ''
     ret = {}
-    cmd = 'pacman -Qqo {0!r}'
+    cmd_prefix = ['pacman', '-Qqo']
+
     for path in paths:
-        ret[path] = __salt__['cmd.run_stdout'](cmd.format(path))
+        ret[path] = __salt__['cmd.run_stdout'](cmd_prefix + [path],
+                                               python_shell=False)
     if len(ret) == 1:
         return next(six.itervalues(ret))
     return ret
