@@ -5,10 +5,13 @@ Support for Tomcat
 This module uses the manager webapp to manage Apache tomcat webapps.
 If the manager webapp is not configured some of the functions won't work.
 
-.. note::
-
-    The config format was changed in 2014.7.0, but backwards compatibility for
-    the old-style config will be in the 2014.7.1 release.
+:configuration:
+    - Java bin path should be in default path
+    - If ipv6 is enabled make sure you permit manager access to ipv6 interface
+      "0:0:0:0:0:0:0:1"
+    - If you are using tomcat.tar.gz it has to be installed or symlinked under
+      ``/opt``, preferably using name tomcat
+    - "tomcat.signal start/stop" works but it does not use the startup scripts
 
 The following grains/pillar should be set:
 
@@ -60,11 +63,11 @@ Also configure a user in the conf/tomcat-users.xml file:
 from __future__ import absolute_import
 
 # Import python libs
+import os
+import re
 import glob
 import hashlib
 import tempfile
-import os
-import re
 import logging
 
 # Import 3rd-party libs
@@ -108,7 +111,7 @@ def __virtual__():
     '''
     if __catalina_home() or _auth('dummy'):
         return 'tomcat'
-    return False
+    return (False, 'Tomcat execution module not loaded: neither Tomcat installed locally nor tomcat-manager credentials set in grains/pillar/config.')
 
 
 def __catalina_home():
@@ -172,12 +175,23 @@ def _auth(uri):
     return _build_opener(basic, digest)
 
 
-def _extract_version(war):
+def _extract_war_version(war):
     '''
-    extract the version from the war name
+    Extract the version from the war file name.  There does not seem to be a
+    standard for encoding the version into the `war file name
+    <https://tomcat.apache.org/tomcat-6.0-doc/deployer-howto.html>`_.
+
+    Examples:
+
+    .. code-block::
+
+        /path/salt-2015.8.6.war -> 2015.8.6
+        /path/V6R2013xD5.war -> V6R2013xD5
     '''
-    version = re.findall("-([\\d.-]+)$", os.path.basename(war).replace('.war', ''))
-    return version[0] if len(version) == 1 else None
+    basename = os.path.basename(war)
+    war_package = os.path.splitext(basename)[0]  # remove '.war'
+    version = re.findall("-([\\d.-]+)$", war_package)  # try semver
+    return version[0] if version and len(version) == 1 else war_package  # default to whole name
 
 
 def _wget(cmd, opts=None, url='http://localhost:8080/manager', timeout=180):
@@ -511,7 +525,8 @@ def deploy_war(war,
                saltenv='base',
                timeout=180,
                env=None,
-               temp_war_location=None):
+               temp_war_location=None,
+               version=''):
     '''
     Deploy a WAR file
 
@@ -532,6 +547,17 @@ def deploy_war(war,
     temp_war_location : None
         use another location to temporarily copy to war file
         by default the system's temp directory is used
+    version : ''
+        Specify the war version.  If this argument is provided, it overrides
+        the version encoded in the war file name, if one is present.
+
+        Examples:
+
+        .. code-block:: bash
+
+            salt '*' tomcat.deploy_war salt://salt-2015.8.6.war version=2015.08.r6
+
+        .. versionadded:: 2015.8.6
 
     CLI Examples:
 
@@ -553,9 +579,9 @@ def deploy_war(war,
     '''
     if env is not None:
         salt.utils.warn_until(
-            'Boron',
+            'Carbon',
             'Passing a salt environment should be done using \'saltenv\' '
-            'not \'env\'. This functionality will be removed in Salt Boron.'
+            'not \'env\'. This functionality will be removed in Salt Carbon.'
         )
         # Backwards compatibility
         saltenv = env
@@ -583,13 +609,11 @@ def deploy_war(war,
     else:
         tfile = war
 
-    version_string = _extract_version(war)
-
     # Prepare options
     opts = {
         'war': 'file:{0}'.format(tfile),
         'path': context,
-        'version': version_string,
+        'version': version or _extract_war_version(war),
     }
     if force == 'yes':
         opts['update'] = 'true'
@@ -607,7 +631,7 @@ def deploy_war(war,
 
 def passwd(passwd,
            user='',
-           alg='md5',
+           alg='sha1',
            realm=None):
     '''
     This function replaces the $CATALINA_HOME/bin/digest.sh script
@@ -622,23 +646,15 @@ def passwd(passwd,
         salt '*' tomcat.passwd secret tomcat sha1
         salt '*' tomcat.passwd secret tomcat sha1 'Protected Realm'
     '''
-    if alg == 'md5':
-        m = hashlib.md5()
-    elif alg == 'sha1':
-        m = hashlib.sha1()
-    else:
-        return False
+    # Shouldn't it be SHA265 instead of SHA1?
+    digest = hasattr(hashlib, alg) and getattr(hashlib, alg) or None
+    if digest:
+        if realm:
+            digest.update('{0}:{1}:{2}'.format(user, realm, passwd,))
+        else:
+            digest.update(passwd)
 
-    if realm:
-        m.update('{0}:{1}:{2}'.format(
-            user,
-            realm,
-            passwd,
-            ))
-    else:
-        m.update(passwd)
-
-    return m.hexdigest()
+    return digest and digest.hexdigest() or False
 
 
 # Non-Manager functions
