@@ -33,7 +33,17 @@ config:
     myprofile:
         keyid: GKTADJGHEIQSXMKKRBJ08H
         key: askdjghsdfjkghWupUjasdflkdfklgjsdfjajkghs
-            region: us-east-1
+        region: us-east-1
+
+.. code-block:: yaml
+
+    aws:
+        region:
+            us-east-1:
+                profile:
+                    keyid: GKTADJGHEIQSXMKKRBJ08H
+                    key: askdjghsdfjkghWupUjasdflkdfklgjsdfjajkghs
+                    region: us-east-1
 
 .. code-block:: yaml
 
@@ -52,16 +62,14 @@ config:
             - vpc_id: vpc-123456
             - cidr_block: 10.0.0.0/16
             - region: us-east-1
-            - keyid: GKTADJGHEIQSXMKKRBJ08H
-            - key: askdjghsdfjkghWupUjasdflkdfklgjsdfjajkghs
+            - profile: myprofile
 
+    {% set profile = salt['pillar.get']('aws:region:us-east-1:profile' ) %}
     Ensure internet gateway exists:
         boto_vpc.internet_gateway_present:
             - name: myigw
             - vpc_name: myvpc
-            - region: us-east-1
-            - keyid: GKTADJGHEIQSXMKKRBJ08H
-            - key: askdjghsdfjkghWupUjasdflkdfklgjsdfjajkghs
+            - profile: {{ profile }}
 
     Ensure route table exists:
         boto_vpc.route_table_present:
@@ -70,13 +78,13 @@ config:
             - routes:
               - destination_cidr_block: 0.0.0.0/0
                 instance_id: i-123456
-                interface_id: eni-123456
-            - subnets:
-              - name: subnet1
-              - name: subnet2
+            - subnet_names:
+              - subnet1
+              - subnet2
             - region: us-east-1
-            - keyid: GKTADJGHEIQSXMKKRBJ08H
-            - key: askdjghsdfjkghWupUjasdflkdfklgjsdfjajkghs
+            - profile:
+                keyid: GKTADJGHEIQSXMKKRBJ08H
+                key: askdjghsdfjkghWupUjasdflkdfklgjsdfjajkghs
 '''
 
 # Import Python Libs
@@ -154,14 +162,15 @@ def present(name, cidr_block, instance_tenancy=None, dns_support=None,
             ret['comment'] = 'VPC {0} is set to be created.'.format(name)
             ret['result'] = None
             return ret
-        r = __salt__['boto_vpc.create'](cidr_block, instance_tenancy,
-                                        name, dns_support, dns_hostnames,
-                                        tags, region, key, keyid, profile)
+        r = __salt__['boto_vpc.create'](cidr_block, instance_tenancy=instance_tenancy, vpc_name=name,
+                                        enable_dns_support=dns_support, enable_dns_hostnames=dns_hostnames,
+                                        tags=tags, region=region, key=key, keyid=keyid,
+                                        profile=profile)
         if not r.get('created'):
             ret['result'] = False
             ret['comment'] = 'Failed to create VPC: {0}.'.format(r['error']['message'])
             return ret
-        _describe = __salt__['boto_vpc.describe'](r['id'], region=region, key=key,
+        _describe = __salt__['boto_vpc.describe'](vpc_id=r['id'], region=region, key=key,
                                                   keyid=keyid, profile=profile)
         ret['changes']['old'] = {'vpc': None}
         ret['changes']['new'] = _describe
@@ -217,7 +226,7 @@ def absent(name, tags=None, region=None, key=None, keyid=None, profile=None):
         ret['comment'] = 'VPC {0} is set to be removed.'.format(name)
         ret['result'] = None
         return ret
-    r = __salt__['boto_vpc.delete'](name=name, tags=tags,
+    r = __salt__['boto_vpc.delete'](vpc_name=name, tags=tags,
                                     region=region, key=key,
                                     keyid=keyid, profile=profile)
     if not r['deleted']:
@@ -230,6 +239,199 @@ def absent(name, tags=None, region=None, key=None, keyid=None, profile=None):
     return ret
 
 
+def dhcp_options_present(name, dhcp_options_id=None, vpc_name=None, vpc_id=None,
+                         domain_name=None, domain_name_servers=None, ntp_servers=None,
+                         netbios_name_servers=None, netbios_node_type=None,
+                         tags=None, region=None, key=None, keyid=None, profile=None):
+    '''
+    Ensure a set of DHCP options with the given settings exist.
+    Note that the current implementation only SETS values during option set
+    creation.  It is unable to update option sets in place, and thus merely
+    verifies the set exists via the given name and/or dhcp_options_id param.
+
+    name
+        (string)
+        Name of the DHCP options.
+
+    vpc_name
+        (string)
+        Name of a VPC to which the options should be associated.  Either
+        vpc_name or vpc_id must be provided.
+
+    vpc_id
+        (string)
+        Id of a VPC to which the options should be associated.  Either
+        vpc_name or vpc_id must be provided.
+
+    domain_name
+        (string)
+        Domain name to be assiciated with this option set.
+
+    domain_name_servers
+        (list of strings)
+        The IP address(es) of up to four domain name servers.
+
+    ntp_servers
+        (list of strings)
+        The IP address(es) of up to four desired NTP servers.
+
+    netbios_name_servers
+        (list of strings)
+        The IP address(es) of up to four NetBIOS name servers.
+
+    netbios_node_type
+        (string)
+        The NetBIOS node type (1, 2, 4, or 8).  For more information about
+        the allowed values, see RFC 2132.  The recommended is 2 at this
+        time (broadcast and multicast are currently not supported).
+
+    tags
+        (dict of key:value pairs)
+        A set of tags to be added.
+
+    region
+        (string)
+        Region to connect to.
+
+    key
+        (string)
+        Secret key to be used.
+
+    keyid
+        (string)
+        Access key to be used.
+
+    profile
+        (various)
+        A dict with region, key and keyid, or a pillar key (string) that
+        contains a dict with region, key and keyid.
+
+    .. versionadded:: 2016.3.0
+    '''
+    ret = {'name': name,
+           'result': True,
+           'comment': '',
+           'changes': {}
+           }
+    _new = {'domain_name': domain_name,
+            'domain_name_servers': domain_name_servers,
+            'ntp_servers': ntp_servers,
+            'netbios_name_servers': netbios_name_servers,
+            'netbios_node_type': netbios_node_type
+           }
+
+    # boto provides no "update_dhcp_options()" functionality, and you can't delete it if
+    # it's attached, and you can't detach it if it's the only one, so just check if it's
+    # there or not, and make no effort to validate it's actual settings... :(
+    ### TODO - add support for multiple sets of DHCP options, and then for "swapping out"
+    ###        sets by creating new, mapping, then deleting the old.
+    r = __salt__['boto_vpc.dhcp_options_exists'](dhcp_options_id=dhcp_options_id,
+                                                 dhcp_options_name=name,
+                                                 region=region, key=key, keyid=keyid,
+                                                 profile=profile)
+    if 'error' in r:
+        ret['result'] = False
+        ret['comment'] = 'Failed to validate DHCP options: {0}.'.format(r['error']['message'])
+        return ret
+
+    if r.get('exists'):
+        ret['comment'] = 'DHCP options already present.'
+        return ret
+    else:
+        if __opts__['test']:
+            ret['comment'] = 'DHCP options {0} are set to be created.'.format(name)
+            ret['result'] = None
+            return ret
+
+        r = __salt__['boto_vpc.create_dhcp_options'](domain_name=domain_name,
+                                                     domain_name_servers=domain_name_servers,
+                                                     ntp_servers=ntp_servers,
+                                                     netbios_name_servers=netbios_name_servers,
+                                                     netbios_node_type=netbios_node_type,
+                                                     dhcp_options_name=name, tags=tags,
+                                                     vpc_id=vpc_id, vpc_name=vpc_name,
+                                                     region=region, key=key, keyid=keyid,
+                                                     profile=profile)
+        if not r.get('created'):
+            ret['result'] = False
+            ret['comment'] = 'Failed to create DHCP options: {1}'.format(r['error']['message'])
+            return ret
+
+        ret['changes']['old'] = {'dhcp_options': None}
+        ret['changes']['new'] = {'dhcp_options': _new}
+        ret['comment'] = 'DHCP options {0} created.'.format(name)
+        return ret
+
+
+def dhcp_options_absent(name=None, dhcp_options_id=None, region=None, key=None, keyid=None, profile=None):
+    '''
+    Ensure a set of DHCP options with the given settings exist.
+
+    name
+        (string)
+        Name of the DHCP options set.
+
+    dhcp_options_id
+        (string)
+        Id of the DHCP options set.
+
+    region
+        (string)
+        Region to connect to.
+
+    key
+        (string)
+        Secret key to be used.
+
+    keyid
+        (string)
+        Access key to be used.
+
+    profile
+        (various)
+        A dict with region, key and keyid, or a pillar key (string) that
+        contains a dict with region, key and keyid.
+
+    .. versionadded:: 2016.3.0
+    '''
+    ret = {'name': name,
+           'result': True,
+           'comment': '',
+           'changes': {}
+           }
+
+    r = __salt__['boto_vpc.get_resource_id']('dhcp_options', name=name,
+                                             region=region, key=key,
+                                             keyid=keyid, profile=profile)
+    if 'error' in r:
+        ret['result'] = False
+        ret['comment'] = 'Failed to delete DHCP options: {0}.'.format(r['error']['message'])
+        return ret
+
+    _id = r.get('id')
+
+    if not _id:
+        ret['comment'] = 'DHCP options {0} do not exist.'.format(name)
+        return ret
+
+    if __opts__['test']:
+        ret['comment'] = 'DHCP options {0} are set to be deleted.'.format(name)
+        ret['result'] = None
+        return ret
+
+    r = __salt__['boto_vpc.delete_dhcp_options'](dhcp_options_id=r['id'], region=region,
+                                                 key=key, keyid=keyid, profile=profile)
+    if not r.get('deleted'):
+        ret['result'] = False
+        ret['comment'] = 'Failed to delete DHCP options: {0}'.format(r['error']['message'])
+        return ret
+
+    ret['changes']['old'] = {'dhcp_options': _id}
+    ret['changes']['new'] = {'dhcp_options': None}
+    ret['comment'] = 'DHCP options {0} deleted.'.format(name)
+    return ret
+
+
 def subnet_present(name, cidr_block, vpc_name=None, vpc_id=None,
                    availability_zone=None, tags=None, region=None,
                    key=None, keyid=None, profile=None):
@@ -302,7 +504,7 @@ def subnet_present(name, cidr_block, vpc_name=None, vpc_id=None,
             ret['result'] = False
             ret['comment'] = 'Failed to create subnet: {0}'.format(r['error']['message'])
             return ret
-        _describe = __salt__['boto_vpc.describe_subnet'](r['id'], region=region, key=key,
+        _describe = __salt__['boto_vpc.describe_subnet'](subnet_id=r['id'], region=region, key=key,
                                                          keyid=keyid, profile=profile)
         ret['changes']['old'] = {'subnet': None}
         ret['changes']['new'] = _describe
@@ -510,8 +712,7 @@ def route_table_present(name, vpc_name=None, vpc_id=None, routes=None,
     '''
     Ensure route table with routes exists and is associated to a VPC.
 
-
-    Example::
+    Example:
 
     .. code-block:: yaml
 
@@ -520,8 +721,13 @@ def route_table_present(name, vpc_name=None, vpc_id=None, routes=None,
             - vpc_id: vpc-123456
             - routes:
               - destination_cidr_block: 0.0.0.0/0
+                internet_gateway_name: InternetGateway
+              - destination_cidr_block: 10.10.11.0/24
                 instance_id: i-123456
+              - destination_cidr_block: 10.10.12.0/24
                 interface_id: eni-123456
+              - destination_cidr_block: 10.10.13.0/24
+                instance_name: mygatewayserver
             - subnet_names:
               - subnet1
               - subnet2
@@ -537,7 +743,7 @@ def route_table_present(name, vpc_name=None, vpc_id=None, routes=None,
         Either vpc_name or vpc_id must be provided.
 
     routes
-        A list of routes.
+        A list of routes.  Each route has a cidr and a target.
 
     subnet_ids
         A list of subnet ids to associate
@@ -576,16 +782,17 @@ def route_table_present(name, vpc_name=None, vpc_id=None, routes=None,
         ret['result'] = _ret['result']
         if ret['result'] is False:
             return ret
-    _ret = _routes_present(route_table_name=name, routes=routes, tags=tags, region=region, key=key,
-                           keyid=keyid, profile=profile)
+    _ret = _routes_present(route_table_name=name, routes=routes, tags=tags,
+                           region=region, key=key, keyid=keyid, profile=profile)
     ret['changes'] = dictupdate.update(ret['changes'], _ret['changes'])
     ret['comment'] = ' '.join([ret['comment'], _ret['comment']])
     if not _ret['result']:
         ret['result'] = _ret['result']
         if ret['result'] is False:
             return ret
-    _ret = _subnets_present(route_table_name=name, subnet_ids=subnet_ids, subnet_names=subnet_names, tags=tags, region=region, key=key,
-                            keyid=keyid, profile=profile)
+    _ret = _subnets_present(route_table_name=name, subnet_ids=subnet_ids,
+                            subnet_names=subnet_names, tags=tags, region=region,
+                            key=key, keyid=keyid, profile=profile)
     ret['changes'] = dictupdate.update(ret['changes'], _ret['changes'])
     ret['comment'] = ' '.join([ret['comment'], _ret['comment']])
     if not _ret['result']:
@@ -595,14 +802,16 @@ def route_table_present(name, vpc_name=None, vpc_id=None, routes=None,
     return ret
 
 
-def _route_table_present(name, vpc_name=None, vpc_id=None, tags=None, region=None, key=None, keyid=None, profile=None):
+def _route_table_present(name, vpc_name=None, vpc_id=None, tags=None, region=None,
+                         key=None, keyid=None, profile=None):
     ret = {'name': name,
            'result': True,
            'comment': '',
            'changes': {}
            }
 
-    r = __salt__['boto_vpc.get_resource_id'](resource='route_table', name=name, region=region, key=key, keyid=keyid,
+    r = __salt__['boto_vpc.get_resource_id'](resource='route_table', name=name,
+                                             region=region, key=key, keyid=keyid,
                                              profile=profile)
     if 'error' in r:
         ret['result'] = False
@@ -618,8 +827,11 @@ def _route_table_present(name, vpc_name=None, vpc_id=None, tags=None, region=Non
             ret['result'] = None
             return ret
 
-        r = __salt__['boto_vpc.create_route_table'](route_table_name=name, vpc_name=vpc_name, vpc_id=vpc_id, tags=tags,
-                                                    region=region, key=key, keyid=keyid, profile=profile)
+        r = __salt__['boto_vpc.create_route_table'](route_table_name=name,
+                                                    vpc_name=vpc_name,
+                                                    vpc_id=vpc_id, tags=tags,
+                                                    region=region, key=key,
+                                                    keyid=keyid, profile=profile)
         if not r.get('created'):
             ret['result'] = False
             ret['comment'] = 'Failed to create route table: {0}.'.format(r['error']['message'])
@@ -640,8 +852,8 @@ def _routes_present(route_table_name, routes, tags=None, region=None, key=None,
            'changes': {}
            }
 
-    route_table = __salt__['boto_vpc.describe_route_table'](route_table_name=route_table_name, tags=tags, region=region,
-                                                            key=key, keyid=keyid, profile=profile)
+    route_table = __salt__['boto_vpc.describe_route_table'](route_table_name=route_table_name, tags=tags,
+                                                            region=region, key=key, keyid=keyid, profile=profile)
     if 'error' in route_table:
         msg = 'Could not retrieve configuration for route table {0}: {1}`.'.format(route_table_name,
                                                                                    route_table['error']['message'])
@@ -669,6 +881,17 @@ def _routes_present(route_table_name, routes, tags=None, region=None, key=None,
                     ret['result'] = False
                     return ret
                 _r['gateway_id'] = r['id']
+            if i.get('instance_name'):
+                running_states = ('pending', 'rebooting', 'running', 'stopping', 'stopped')
+                r = __salt__['boto_ec2.get_id'](name=i['instance_name'], region=region,
+                                                key=key, keyid=keyid, profile=profile,
+                                                in_states=running_states)
+                if r is None:
+                    msg = 'Instance {0} does not exist.'.format(i['instance_name'])
+                    ret['comment'] = msg
+                    ret['result'] = False
+                    return ret
+                _r['instance_id'] = r
             _routes.append(_r)
 
     to_delete = []
